package lab.game;

import java.io.Serial;
import java.util.ArrayList;
import java.util.List;

import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;

public class BulletAnimated extends Bullet {

	@Serial
	private static final long serialVersionUID = 4366455699671618979L;
	
	private static final double SIZE = 40;
	private Cannon cannon;
	private static Image image = new Image(BulletAnimated.class.getResourceAsStream("fireball-transparent.gif"));
	
	private transient List<HitListener> hitListeners = new ArrayList<>();

	public BulletAnimated(World world, Cannon cannon, MyPoint position, MyPoint velocity, MyPoint acceleration) {
		super(world, position, velocity, acceleration);
		this.cannon = cannon;
	}

	@Override
	public void drawInternal(GraphicsContext gc) {
		gc.drawImage(image, getPosition().getX(), getPosition().getY(), SIZE, SIZE);
		gc.strokeRect(position.getX(), position.getY(), SIZE, SIZE);
	}

	@Override
	public void hitBy(Collisionable another) {
		if (another instanceof Ufo) {
			world.remove(this);
			fireUfoDestroyed();
		}
	}

	public void reload() {
		position = cannon.getPosition();
		velocity = new MyPoint(0, 0);
	}

	public boolean addHitListener(HitListener e) {
		return hitListeners.add(e);
	}

	public boolean removeHitListener(HitListener o) {
		return hitListeners.remove(o);
	}

	private void fireUfoDestroyed() {
		for (HitListener hitListener : hitListeners) {
			hitListener.ufoDestroyed();
		}
	}

}