package lab.gui;

import java.io.IOException;
import java.net.URL;

import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonType;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.stage.WindowEvent;
import lab.Setting;
import lab.data.Score;
import lab.storage.FileStorage;
import lombok.extern.log4j.Log4j2;

/**
 * Class <b>App</b> - extends class Application and it is an entry point of the
 * program
 * 
 * @author Java I
 */
@Log4j2
public class App extends Application {

	private GameController gameController;

	private boolean viewMode;

	private Stage primaryStage;

	public static void main(String[] args) {
		log.info("Application lauched");
		Setting.configure(Setting.builder().scoreStorageInterface(new FileStorage()).build());

		launch(args);
	}

	@Override
	public void start(Stage primaryStage) {
		try {
			// Construct a main window with a canvas.
			Alert alert = new Alert(AlertType.CONFIRMATION, "View Mode?", ButtonType.YES, ButtonType.NO);
			alert.showAndWait().ifPresent(button -> viewMode = ButtonType.YES.equals(button));
			FXMLLoader gameLoader = new FXMLLoader(getClass().getResource("/lab/gui/gameWindow.fxml"));
			Parent root = gameLoader.load();
			gameController = gameLoader.getController();
			gameController.setViewMode(viewMode);
			gameController.setApp(this);
			Scene scene = new Scene(root);
			this.primaryStage = primaryStage;
			primaryStage.setScene(scene);
			primaryStage.setTitle("Java 2 - 2nd laboratory");
			primaryStage.show();
			// Exit program when main window is closed
			primaryStage.setOnCloseRequest(this::exitProgram);
		} catch (Exception e) {
			log.error("Error during game play.", e);
		}
	}

	public Stage createDialogStage(Score score) throws IOException {
		Stage dialog = new Stage();
		dialog.initStyle(StageStyle.UTILITY);
		FXMLLoader editLoader = new FXMLLoader(getClass().getResource("/lab/gui/edit.fxml"));
		Parent root = editLoader.load();
		EditController editController = editLoader.getController();
		editController.setStage(dialog);
		editController.setObjectToEdit(score);
		editController.setGameController(gameController);
		Scene scene = new Scene(root);
		URL cssUrl = getClass().getResource("application.css");
		scene.getStylesheets().add(cssUrl.toString());
		dialog.setScene(scene);
		dialog.initModality(Modality.APPLICATION_MODAL);
		dialog.initOwner(primaryStage);
		return dialog;

	}

	@Override
	public void stop() throws Exception {
		gameController.stop();
		super.stop();
		log.info("Gamne stoped");
	}

	private void exitProgram(WindowEvent evt) {
		if (gameController != null) {
			gameController.stop();
		}
		Setting.getInstance().getScoreStorageInterface().stop();
		log.info("Exiting game");
		System.exit(0);
	}
}